// Copyright (C) 2023 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:flutter/material.dart';
import 'package:gap/gap.dart';
import 'package:wyatt_component_copy_with_extension/wyatt_component_copy_with_extension.dart';
import 'package:wyatt_ui_components/wyatt_ui_components.dart';
import 'package:wyatt_ui_kit/src/components/cards/portfolio_card/widgets/portfolio_card_header.dart';
import 'package:wyatt_ui_kit/src/components/cards/portfolio_card/widgets/portfolio_card_images.dart';
import 'package:wyatt_ui_kit/src/components/cards/widgets/card_text.dart';
import 'package:wyatt_ui_kit/src/components/cards/widgets/card_wrapper.dart';

part 'portfolio_card.g.dart';

const _spacing = 20.0;

@ComponentCopyWithExtension()
class PortfolioCard extends PortfolioCardComponent with $PortfolioCardCWMixin {
  const PortfolioCard({
    super.showAssetsOnTop,
    super.keywords,
    super.keywordsBackgroundColors,
    super.description,
    super.logo,
    super.projectName,
    super.subtitle,
    super.ctas,
    super.assets,
    super.radius,
    super.padding,
    super.borderColors,
    super.backgroundColors,
    super.stroke,
    super.minSize,
    super.maxSize,
    super.shadow,
    super.titleStyle,
    super.subtitleStyle,
    super.bodyStyle,
    super.background,
    super.key,
  });

  @override
  Widget build(BuildContext context) {
    final themeTitleStyle = ThemeHelper.maybeGetElement<TextStyle, TextStyle>(
      [
        titleStyle,
        Theme.of(context).extension<CardThemeExtension>()?.titleStyle,
        CardThemeExtensionDefault.from(Theme.of(context)).titleStyle,
      ],
    );

    final themeSubtitleStyle =
        ThemeHelper.maybeGetElement<TextStyle, TextStyle>(
      [
        subtitleStyle,
        Theme.of(context).extension<CardThemeExtension>()?.subtitleStyle,
        CardThemeExtensionDefault.from(Theme.of(context)).subtitleStyle,
      ],
    );

    final themeBodyStyle = ThemeHelper.maybeGetElement<TextStyle, TextStyle>(
      [
        bodyStyle,
        Theme.of(context).extension<CardThemeExtension>()?.bodyStyle,
        CardThemeExtensionDefault.from(Theme.of(context)).bodyStyle,
      ],
    );

    final themeBorderRadius =
        ThemeHelper.maybeGetElement<BorderRadiusGeometry, BorderRadiusGeometry>(
      [
        radius,
        Theme.of(context).extension<CardThemeExtension>()?.radius,
        CardThemeExtensionDefault.from(Theme.of(context)).radius,
      ],
    );

    final themeMaxSize = ThemeHelper.maybeGetElement<Size, Size>(
      [
        maxSize,
        Theme.of(context).extension<CardThemeExtension>()?.maxSize,
        CardThemeExtensionDefault.from(Theme.of(context)).maxSize,
      ],
    );

    return CardWrapper(
      background: background,
      padding: padding,
      radius: radius,
      backgroundColors: backgroundColors,
      borderColors: borderColors,
      shadow: shadow,
      stroke: stroke,
      maxSize: maxSize,
      minSize: minSize,
      child: Column(
        mainAxisSize: MainAxisSize.min,
        children: [
          if (showAssetsOnTop ?? false) ...[
            if (assets != null) ...[
              PortfolioCardImages(
                assets: assets,
                radius: themeBorderRadius,
                maxSize: themeMaxSize,
              ),
              const Gap(_spacing),
            ],
            CardText.fromWrapper(
              description!,
              style: themeBodyStyle,
            ),
            const Gap(_spacing),
            PortfolioCardHeader(
              logo: logo,
              projectName: projectName,
              subtitle: subtitle,
              keywords: keywords,
              keywordsBackgroundColors: keywordsBackgroundColors,
              titleStyle: themeTitleStyle,
              subtitleStyle: themeSubtitleStyle,
              keywordsStyle: themeBodyStyle,
            ),
          ] else ...[
            PortfolioCardHeader(
              logo: logo,
              projectName: projectName,
              subtitle: subtitle,
              keywords: keywords,
              keywordsBackgroundColors: keywordsBackgroundColors,
              titleStyle: themeTitleStyle,
              subtitleStyle: themeSubtitleStyle,
              keywordsStyle: themeBodyStyle,
            ),
            const Gap(10),
            if (assets != null) ...[
              PortfolioCardImages(
                assets: assets,
                radius: themeBorderRadius,
                maxSize: themeMaxSize,
              ),
              const Gap(_spacing),
            ],
            CardText.fromWrapper(
              description!,
              style: themeBodyStyle,
            ),
          ],
          if (ctas != null)
            ...[const Gap(_spacing), ...ctas!].map(
              (e) => SelectionContainer.disabled(child: e),
            ),
        ],
      ),
    );
  }
}
